// AP(r) Computer Science Marine Biology Simulation:
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.util.Random;
import java.awt.Color;
import java.util.ArrayList;

/**
 * Class that represents a starfish.  These
 * animals live on the bottom of the ocean. 
 * (They are not fish).  If they aren't at the bottom
 * they will try to sink to the bottom.  
 * 
 * This class was created
 * to show how to extend the Marine Biology Case
 * Study past Fish. 
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 * Georgia Tech Jan 2005
 */
public class Starfish extends AbstractActionable
{
  //////////////// fields //////////////////////////
  
  // Class Variable: Shared among ALL Starfish
  private static int nextAvailableID = 1;   // next avail id
  
  //////////////// constructors ////////////////////
  
  /**
   * Constructor that takes an environment and a location
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   */
  public Starfish(Environment theEnv, Location theLoc)
  {
    // call the other constructor with a random direction
    this(theEnv,theLoc,theEnv.randomDirection());
  }
  
  /**
   * Constructor that takes an environment, location, and 
   * direction
   * @param theEnv  the environment this object is in
   * @param theLoc  the location of this object
   * @param theDir the direction this object is facing
   */
  public Starfish(Environment theEnv, Location theLoc, 
                  Direction theDir)
  {
    // let the parent constructor handle this
    super(nextAvailableID,theEnv,theLoc,theDir);
    nextAvailableID++; // increment the next id
  }
  
  //////////////// methods /////////////////////////
  
  /**
   * Method to tell the object to act
   */
  public void act()
  {
    // check that this object is still in the environment
    if (isInEnv())
    {
      // try to move
      move();
    }
  }
  
  /**
   * Method to try to move this object
   */
  public void move()
  {
    
    // get the max y 
    int maxY = environment().numRows() - 1;
    
    // declare variables
    Location[] locArray = null;
    Location nextLoc = null;
    
    // if at max y row (bottom) try to go east or west
    if (maxY == location().row())
      locArray = getEastAndWestEmptyNeighbors();
    
    // else go south, southeast, or southwest
    else
      locArray = getSouthEmptyNeighbors();
    
    // if no empty neighbors stay where you are
    if (locArray[0] == null)
    {
      Debug.println("no empty neighbors");
      nextLoc = location();
    }
    // take first empty neighbor
    else
    {
      Debug.println("are empty neighbors");
      nextLoc = locArray[0];
    }
    
    // move to the next location
    move(nextLoc);
    
  }
}
